import sys
import help
import rawpy
import imageio
import os
import shutil
import json
from PIL import Image
from jinja2 import Environment, FileSystemLoader
import subprocess

with open('config.json', 'r') as file:
    config: dict = json.load(file)

# Initialiser l'environnement avec un dossier de templates
env = Environment(loader=FileSystemLoader('src/templates'))

template = env.get_template('page.html')

def get_files_recursive(directory):
    file_paths = []
    for root, dirs, files in os.walk(directory):
        for file in files:
            file_paths.append(os.path.join(root, file))
    return file_paths


def move_files(files: list[str], input_path, output_path):
    for key, type_file_extensions in config["file_type_extensions"].items():
        for file in files:
            for type_file_extension in type_file_extensions:
                if file.endswith(type_file_extension):
                    path = file.replace(input_path, output_path)[:-len(type_file_extension)]
                    if not os.path.exists(path):
                        if (key not in ["raw", "large", "small"]):
                            continue
                        os.makedirs(path)
                    shutil.copy2(file, path)

def nef_to_jpeg(path:str):
    with rawpy.imread(path) as raw:
        rgb = raw.postprocess()
        imageio.imwrite(path.replace('.NEF', '.jpeg'), rgb)

def png_to_jpeg(path: str):
    im = Image.open(path)
    rgb_im = im.convert('RGB')
    rgb_im.save(path.replace('.nef', '.jpeg'))


def generate_missing_images(raws: list[str], large: list[str], small: list[str]):
    if len(small) == 0:
        if (len(large) == 0):
            nef_to_jpeg(raws[0])
            return
        png_to_jpeg(large[0])
        return

def get_exif(raws: list[str]):
    if (len(raws) == 0):
        return None
    return str(subprocess.check_output(["exiftool", f"{raws[0]}"])).replace("\\n", "\n")


def generate_page(output_path: str):
    folders = os.listdir(output_path)
    if "general" in folders:
        folders.remove("general")
    for folder in folders:
        path = os.path.join(output_path, folder)
        if (os.path.isdir(path)):
            files: list[str] = [os.path.join(path, file) for file in os.listdir(path) if os.path.isfile(os.path.join(path, file))]
            
            raws = [file for ext in config["file_type_extensions"]["raw"] for file in files if file.endswith(ext)]
            larges = [file for ext in config["file_type_extensions"]["large"] for file in files if file.endswith(ext)]
            smalls = [file for ext in config["file_type_extensions"]["small"] for file in files if file.endswith(ext)]
            
            print(folder)
            generate_missing_images(raws, larges, smalls)
            
            exif = get_exif(raws)
            raw = os.path.basename(raws[0]) if len(raws) > 0 else None
            large = os.path.basename(larges[0]) if len(larges) > 0 else None
            small = os.path.basename(smalls[0]) if len(smalls) > 0 else None

            print(f"Generate {folder} html")
            html_rendered = template.render(exif=exif, raw=raw, large=large, small=small)

            with open(os.path.join(path, f"{folder}.html"), "w") as f:
                f.write(html_rendered)

def main():

    if len(sys.argv) < 2:
        help.display(sys.argv[0])
        return

    input_path = sys.argv[1]
    output_path = "output/" if (len(sys.argv) == 2) else sys.argv[2] + "/"

    files: list[str] = get_files_recursive(input_path)

    move_files(files, input_path, output_path)

    generate_page(output_path)

    general_path: str = os.path.join(output_path, "general")
    if not os.path.exists(general_path):
        #os.makedirs(general_path)
        shutil.copytree("src/templates/", general_path)

if (__name__ == "__main__"):
    main()